/**
 * [Kwok GO] Ver:1.0
 * Created Time: 2022-01-20 19:10:42
 * LastEditors: Kwok
 * go mod tidy 清除无用的依赖
 * go mod download 自动下载依赖
 */
package main

import (
	"fmt"
	"html/template"
	"log"
	"math/rand"
	"net/http"
	"regexp"
	"strings"
	"time"
	"unicode/utf8"

	"github.com/cao-guang/pinyin"
	"github.com/gin-gonic/gin"
	"gorm.io/driver/sqlite"
	"gorm.io/gorm"
)

//检查是否为蜘蛛(未实现)
func checkIsRobots() gin.HandlerFunc {
	return func(c *gin.Context) {
		start := time.Now()
		c.Next()
		cost := time.Since(start)
		log.Println("本次执行耗时", cost)
	}
}

//清除HTML代码
func trimHtml(src string) string {
	re, _ := regexp.Compile("\\<[\\S\\s]+?\\>")
	src = re.ReplaceAllStringFunc(src, strings.ToLower)
	re, _ = regexp.Compile("\\<style[\\S\\s]+?\\</style\\>")
	src = re.ReplaceAllString(src, "")
	re, _ = regexp.Compile("\\<script[\\S\\s]+?\\</script\\>")
	src = re.ReplaceAllString(src, "")
	re, _ = regexp.Compile("\\<[\\S\\s]+?\\>")
	src = re.ReplaceAllString(src, "\n")
	re, _ = regexp.Compile("\\s{2,}")
	src = re.ReplaceAllString(src, "\n")
	return strings.TrimSpace(src)
}

//数据结构
type Enter struct {
	ID           uint   `gorm:"primaryKey"`
	Name         string `gorm:"index;size:80"`
	Shortname    string `gorm:"size:8"`
	Introduction string `gorm:"size:1024"`
	Area         string `gorm:"size:255"`
	Business     string `gorm:"size:2048"`
	Boss         string `gorm:"size:32"`
	Cellphone    string `gorm:"size:15"`
	Email        string `gorm:"size:128"`
	Dateof       string `gorm:"size:32"`
	Address      string `gorm:"size:64"`
}

//随机颜色
func randColor() string {
	var colors = [...]string{"primary", "secondary", "success", "info", "warning", "danger", "light", "dark"}
	color := rand.Perm(len(colors))
	return colors[color[0]]
}
func main() {
	//使用gorm初始化一个sqlite数据库
	db, err := gorm.Open(sqlite.Open("enter.db"), &gorm.Config{})
	if err != nil {
		panic(err) //数据库有问题
	}
	var enterMap Enter            //Enter空实例
	db.AutoMigrate(&enterMap)     //同使用Enter空实例映射数据库结构
	result := db.First(&enterMap) //查询一条数据
	if result.RowsAffected == 0 {
		fmt.Println("数据库里没有数据，写入测试数据", result.RowsAffected)
		var enters = []Enter{{1, "北京鼎盛机电设备维修有限公司", "鼎盛", "<p class=\"txt\">北京鼎盛机电设备维修有限公司位于中国北京，主营业务包括机电设备（电机水泵）维修安装，欢迎新老客户联系我们，咨询交流！&nbsp;&nbsp;&nbsp;我们有好的产品和专业的团队，公司发展迅速，我们为客户提供优质的产品、良好的技术支持、健全的售后服务，北京鼎盛机电设备维修有限公司是机电设备维修,电动机水泵维修,电机水泵销售,机械设备租赁知名企业，如果您对我公司的产品服务有兴趣，请在线留言或者来电咨询。</p>", "北京 房山", "机电设备维修,电动机水泵维修,电机水泵销售,机械设备租赁", "李德才", "13701319230", "", "2017-05-18", "北京市房山区长阳万兴路86号-A583"},
			{2, "深圳市众视显科技有限公司", "众视显", "<p class=\"txt\">深圳市众视显科技有限公司，是一家集生产及销售商用触摸一体机、多媒体广告机、液晶拼接墙等液晶显示终端设备的高新技术企业，公司秉承\"以人为本，科学发展\"的宗旨。以\"客户体验，科技创新”做为核心竟争力。缔造更好的管理团队，技术团队和生产团队,创建和完善、快捷的售后服务体系”，不断满足客户的需求，引进商显市场！<br>\r\n公司拥有自已的生产厂房及新进设备，拥有的人才，基于强大的自主创新能力和对行业需求的深刻理解，深圳市众视显科技有限公司推出了多个应用解决方案，产品广泛应用于广告传媒，金融行业，街头信息查询，医疗事业，交通行业，例如地铁，车站，其他：例如学校，酒店、展会、博物馆等等，给各个行来从方案设计、方案整改、定制生产、安装调试等全过程提供了完整的解决方案支持与服务，赢得了分众传媒，粤海物业等国内企业和各界人士的信任和支持。本着诚实守信的经营原则，愿与商业各界伙伴共同携手，缔造共赢平台，同创美好未来</p>", "广东 深圳", "液晶拼接屏,广告机,液晶广告机,触摸一体机", "何炎", "13145824951", "", "2021-12-28", "深圳市光明区公明街道李松蓢社区第一工业区炮台路48号创新云谷厂房H栋1002"},
			{3, "常州市纽旭机电设备有限公司", "纽旭机电", "位于江苏省常州市武进区，是集产品设计、研发、制造与销售于一体的技术企业。", "江苏 常州", "机电设备维修,电动机水泵维修,电机水泵销售,机械设备租赁", "张先生", "15113534634", "476161917@qq.com", "2016-05-18", "常州市武进区万科城柏悦湾花园21-218号"}}
		db.Create(&enters) //写入数据
	}
	r := gin.Default() //使用Gin默认配置
	r.SetFuncMap(template.FuncMap{
		"randcolor": randColor, //模板里支持randColor函数
	})
	r.Use(checkIsRobots())        //使用中间件
	r.LoadHTMLGlob("templates/*") //载入模板
	r.GET("/", func(c *gin.Context) {
		var enter Enter
		result = db.Order("RANDOM()").Take(&enter) // 随机查询一条
		scheme := "http://"
		if c.Request.TLS != nil {
			scheme = "https://"
		}
		host := c.Request.Host                               //当前域名
		pinyin, err := pinyin.To_Py(enter.Shortname, "", "") //将公司简称转为拼音
		if err != nil {
			fmt.Println("拼音转换出错了：", err)
		}
		cutStr := 80 //不要使用len计算字符长度，会导致Rune越界
		if utf8.RuneCountInString(trimHtml(enter.Introduction+enter.Name)) < cutStr {
			cutStr = utf8.RuneCountInString(trimHtml(enter.Introduction+enter.Name)) - 1
		}
		c.HTML(http.StatusOK, "index.html", gin.H{
			"title":       enter.Business + enter.Name,
			"keywords":    enter.Business + "," + enter.Name,
			"description": string([]rune(trimHtml(enter.Introduction + enter.Name))[:cutStr]), //这里可以写个func来处理更好
			"scheme":      scheme,                                                             //使用的协议
			"host":        host,                                                               //主机(域名)
			"webURL":      scheme + host,                                                      //完整地址
			"business":    strings.Split(enter.Business, ","),                                 //分割字符串
			"intro":       template.HTML(enter.Introduction),                                  //支持html
			"enter":       enter,                                                              //企业信息
			"pinyin":      pinyin,
			"rangeColor":  randColor(),                 //随机颜色
			"copyright":   rand.Intn(2022-2015) + 2015, //返回2015~2022的随机数
		})
	})
	r.Run(":80") //在80端口上运行web服务，这里可以改为8080\9090之类
}
